# -*- coding: utf-8 -*-
from datetime import datetime
import base64
import hashlib
import hmac
from six import text_type
from libthumbor.url_signers import BaseUrlSigner


class UrlSigner(BaseUrlSigner):
    """
    Expects a signature of the form <hmac>:<expiry>, where
       - hmac is a urlsafe base64 encoded HMAC using SHA1.
       - expiry encodes the time in UTC until the url is valid, in the format YYYYMMDDHHMMSS.
       - eg: -8yJNS5QbnIHRqCOTGsqgLnCKk8=:20210326142432
    """

    def validate(self, actual_signature, url):
        if ':' in actual_signature:
            sha1_str, expire_time = actual_signature.split(':')
            if not self._valid_expire_time(expire_time):
                return False

            url_signature = self.signature(expire_time, url)
            return url_signature == sha1_str
        else:
            url_signature = self.signature('', url)
            return url_signature == actual_signature

    def _valid_expire_time(self, expire_time):
        try:
            expiry = datetime.strptime(expire_time, '%Y%m%d%H%M%S')
            return datetime.utcnow() < expiry
        except ValueError:
            return False

    def signature(self, expire_time, url):
        security_key = text_type(self.security_key).encode('utf-8')
        full_url = text_type(expire_time + '/' + url).encode('utf-8')
        return base64.urlsafe_b64encode(hmac.new(security_key, full_url, hashlib.sha1).digest())



from datetime import datetime, timedelta, time, date
import base64
from six import text_type, PY3

from libthumbor.crypto import CryptoURL as ThumborCryptoURL
from libthumbor.url import plain_image_url


class CryptoURL(ThumborCryptoURL):

    def generate_url(self, url, expire = None):
        expire_url = url
        if expire:
            if isinstance(expire, (int, float)):
                expire = datetime.utcnow() + timedelta(seconds=expire)
            elif isinstance(expire, datetime):
                expire = expire
            elif isinstance(expire, date):
                expire = datetime.combine(expire, time.min)
            elif isinstance(expire, time):
                expire = datetime.utcnow() + (datetime.combine(date.min, expire) - datetime.min)
            elif isinstance(expire, timedelta):
                expire = datetime.utcnow() + expire

            expire_str = expire.strftime('%Y%m%d%H%M%S')
            expire_url = '%s/%s' % (expire_str, expire_url)

        _hmac = self.hmac.copy()
        _hmac.update(text_type(expire_url).encode('utf-8'))
        signature = base64.urlsafe_b64encode(_hmac.digest())

        if PY3:
            signature = signature.decode('ascii')

        if expire:
            return '/%s:%s/%s' % (signature, expire_str, url)
        else:
            return '/%s/%s' % (signature, url)

    def generate_new(self, options):
        url = plain_image_url(**options)
        return self.generate_url(url, options.get('expire', None))

    def generate_options(self,
                         image_url,
                         width=0,
                         height=0,
                         smart=False,
                         meta=False,
                         trim=None,
                         adaptive=False,
                         full=False,
                         fit_in=False,
                         horizontal_flip=False,
                         vertical_flip=False,
                         halign='center',
                         valign='middle',
                         expire=None,  #
                         crop_left=None,
                         crop_top=None,
                         crop_right=None,
                         crop_bottom=None,
                         filters=None):
        """
        :param expire: :obj:`int` | :obj:`float` | :obj:`datetime.timedelta` | \
        :obj:`datetime.datetime` | :obj:`datetime.time`, optional
        """
        return self.generate(**{k: v for k, v in locals().items() if v is not None})





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['thumbor_expire']

package_data = \
{'': ['*']}

extras_require = \
{':python_version >= "2" and python_version < "3"': ['libthumbor>=1.3.2,<2.0.0'],
 ':python_version >= "3.6"': ['libthumbor>=2.0.1,<3.0.0']}

setup_kwargs = {
    'name': 'thumbor-expire',
    'version': '0.1.2',
    'description': 'Add timeout verification for thumbor',
    'long_description': '# thumbor-expire\nAdd timeout verification for thumbor\n\n## Use \n\n* Server side\n\nthumbor config file set `URL_SIGNER = thumbor_expire.base64_hmac_sha1_expire`\n\n\n* Client side\n\nuse `from thumbor_expire.crypto import CryptoURL` class and pass expire argument, accept type `:obj:`int` | :obj:`float` | :obj:`datetime.timedelta` | \\\n        :obj:`datetime.datetime` | :obj:`datetime.time`, optional`\n',
    'author': 'HonQii',
    'author_email': 'honqi3014@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/HonQii/thumbor-expire.git',
    'packages': packages,
    'package_data': package_data,
    'extras_require': extras_require,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
}


setup(**setup_kwargs)
